# awl.managment.commands.run_script.py
#
# runs the python script given on the command line within the django env

import imp
from django.core.management.base import BaseCommand

class Command(BaseCommand):
    help = 'Loads (runs) the given python script in the django space'

    def add_arguments(self, parser):
        parser.add_argument('filename', nargs='+', type=str)

    def handle(self, *args, **options):
        imp.load_source('config', options['filename'][0])



from django.conf.urls import include, url
from django.contrib import admin

from awl.rankedmodel import urls as ranked_urls
from awl.tests import views

urlpatterns = [
    url(r'^admin/', include(admin.site.urls)),

    url(r'^rankedmodel/', include(ranked_urls)),

    url(r'^awl_test_views/test_view_for_messages/$', 
        views.test_view_for_messages, ),
]



# awl.utils.py

from django.shortcuts import render
from django.template.loader import render_to_string

# =============================================================================
# Template Methods
# =============================================================================

def render_page(request, page_name, data={}):
    """
    .. deprecated:: 0.12
        Use ``django.shortcuts.render`` instead
    
    This function was a wrapper for ``render_to_response`` that handled
    request context.  The ``django.shortcuts.render`` method does the same
    thing, so this just wraps that now. 
    """
    return render(request, page_name, data)


def render_page_to_string(request, page_name, data={}):
    """A shortcut for using ``render_to_string`` with a
    :class:`RequestContext` automatically.
    """
    return render_to_string(page_name, data, request=request)

# ============================================================================
# Object Model Tools
# ============================================================================

def refetch(obj):
    """Queries the database for the same object that is passed in, refetching
    its contents in case they are stale.

    :param obj:
        Object to refetch

    :returns:
        Refreshed version of the object
    """
    return obj.__class__.objects.get(id=obj.id)


def refetch_for_update(obj):
    """Queries the database for the same object that is passed in, refetching
    its contents and runs ``select_for_update()`` to lock the corresponding
    row until the next commit.

    :param obj:
        Object to refetch
    :returns:
        Refreshed version of the object
    """
    return obj.__class__.objects.select_for_update().get(id=obj.id)

# ============================================================================
# Misc
# ============================================================================

class URLTree(object):
    """A tree representation of the django url regexes.  Each pattern is
    stored in a dictionary with its pattern, full path, name (if available)
    and children.  Root items in the tree can be accessed through the
    ``children`` list attribute.
    """
    def __init__(self):
        self.children = []

        # parse the url pattern tree 
        from django.conf import settings
        urlconf = __import__(settings.ROOT_URLCONF, {}, {}, [''])
        self._parse_node(urlconf.urlpatterns, '', self.children)

    def _parse_node(self, parent, parent_path, children):
        for entry in parent:
            path = parent_path + entry.regex.pattern
            d = {
                'pattern':entry.regex.pattern,
                'path':path,
                'name':entry.name if hasattr(entry, 'name') else '',
                'children':[],
            }
            children.append(d)

            if hasattr(entry, 'url_patterns'):
                self._parse_node(entry.url_patterns, path, d['children'])

    def _depth_traversal(self, node, result):
        text = node['path']
        if node['name']:
            text += ', name=%s' % node['name']

        result.append(text)
        for child in node['children']:
            self._depth_traversal(child, result)

    def as_list(self):
        """Returns a list of strings describing the full paths and patterns
        along with the name of the urls.  Example:

        .. code-block::python
            >>> u = URLTree()
            >>> u.as_list()
            [
                'admin/',
                'admin/$, name=index',
                'admin/login/$, name=login',
            ]
        """
        result = []
        for child in self.children:
            self._depth_traversal(child, result)

        return result

    def print_tree(self):
        """Convenience method for printing the results of
        :class:`URLTree.as_list` to STDOUT
        """
        for line in self.as_list():
            print(line)



# awl.decorators.py
import logging
from functools import wraps

from django.http import Http404

logger = logging.getLogger(__name__)

# ============================================================================
# View Decorators
# ============================================================================

def post_required(method_or_options=[]):
    """View decorator that enforces that the method was called using POST.
    This decorator can be called with or without parameters.  As it is
    expected to wrap a view, the first argument of the method being wrapped is
    expected to be a ``request`` object.

    .. code-block:: python

        @post_required
        def some_view(request):
            pass


        @post_required(['firstname', 'lastname'])
        def some_view(request):
            pass

    The optional parameter contains a single list which specifies the names of
    the expected fields in the POST dictionary.  The list is not exclusive,
    you can pass in fields that are not checked by the decorator.

    :param options:
        List of the names of expected POST keys.
    """
    def decorator(method):
        # handle wrapping or wrapping with arguments; if no arguments (and no
        # calling parenthesis) then method_or_options will be a list,
        # otherwise it will be the wrapped function
        expected_fields = []
        if not callable(method_or_options):
            # not callable means wrapping with arguments
            expected_fields = method_or_options

        @wraps(method)
        def wrapper(*args, **kwargs):
            request = args[0]
            if request.method != 'POST':
                logger.error('POST required for this url')
                raise Http404('only POST allowed for this url')

            missing = []
            for field in expected_fields:
                if field not in request.POST:
                    missing.append(field)

            if missing:
                s = 'Expected fields missing in POST: %s' % missing
                logger.error(s)
                raise Http404(s)

            # everything verified, run the view
            return method(*args, **kwargs)
        return wrapper

    if callable(method_or_options):
        # callable means decorated method without options, call our decorator 
        return decorator(method_or_options)
    return decorator



# awl.tests.test_decorators.py

from django.test import TestCase
from django.http import Http404

from awl.decorators import post_required
from awl.waelsteng import FakeRequest
from wrench.logtools.utils import silence_logging

# ============================================================================

class Worked1(Exception):
    pass

class Worked2(Exception):
    pass


@post_required
def view1(request):
    raise Worked1()


@post_required(['foo'])
def view2(request):
    raise Worked2()

# ============================================================================

class DecoratorTest(TestCase):
    @silence_logging
    def test_post_required(self):
        request = FakeRequest()
        with self.assertRaises(Http404):
            view1(request)

        request = FakeRequest(method='POST')
        with self.assertRaises(Worked1):
            view1(request)

        request = FakeRequest(method='POST', data={ 'foo':'bar', })
        with self.assertRaises(Worked2):
            view2(request)

        request.POST = {
            'bar':'bar',
        }
        with self.assertRaises(Http404):
            view2(request)



import re

PREDEFINED = ('aliceblue', 'antiquewhite', 'aqua', 'aquamarine', 'azure',
    'beige', 'bisque', 'black', 'blanchedalmond', 'blue', 'blueviolet',
    'brown', 'burlywood', 'cadetblue', 'chartreuse', 'chocolate', 'coral',
    'cornflowerblue', 'cornsilk', 'crimson', 'cyan', 'darkblue', 'darkcyan',
    'darkgoldenrod', 'darkgray', 'darkgreen', 'darkkhaki', 'darkmagenta',
    'darkolivegreen', 'darkorange', 'darkorchid', 'darkred', 'darksalmon',
    'darkseagreen', 'darkslateblue', 'darkslategray', 'darkturquoise',
    'darkviolet', 'deeppink', 'deepskyblue', 'dimgray', 'dodgerblue',
    'firebrick', 'floralwhite', 'forestgreen', 'fuchsia', 'gainsboro',
    'ghostwhite', 'gold', 'goldenrod', 'gray', 'green', 'greenyellow',
    'honeydew', 'hotpink', 'indianred', 'indigo', 'ivory', 'khaki',
    'lavender', 'lavenderblush', 'lawngreen', 'lemonchiffon', 'lightblue',
    'lightcoral', 'lightcyan', 'lightgoldenrodyellow', 'lightgray',
    'lightgreen', 'lightpink', 'lightsalmon', 'lightseagreen', 'lightskyblue',
    'lightslategray', 'lightsteelblue', 'lightyellow', 'lime', 'limegreen',
    'linen', 'magenta', 'maroon', 'mediumaquamarine', 'mediumblue',
    'mediumorchid', 'mediumpurple', 'mediumseagreen', 'mediumslateblue',
    'mediumspringgreen', 'mediumturquoise', 'mediumvioletred', 'midnightblue',
    'mintcream', 'mistyrose', 'moccasin', 'navajowhite', 'navy', 'oldlace',
    'olive', 'olivedrab', 'orange', 'orangered', 'orchid', 'palegoldenrod',
    'palegreen', 'paleturquoise', 'palevioletred', 'papayawhip', 'peachpuff',
    'peru', 'pink', 'plum', 'powderblue', 'purple', 'rebeccapurple', 'red',
    'rosybrown', 'royalblue', 'saddlebrown', 'salmon', 'sandybrown',
    'seagreen', 'seashell', 'sienna', 'silver', 'skyblue', 'slateblue',
    'slategray', 'snow', 'springgreen', 'steelblue', 'tan', 'teal', 'thistle',
    'tomato', 'turquoise', 'violet', 'wheat', 'white', 'whitesmoke', 'yellow',
    'yellowgreen'
)


HEX_MATCH = re.compile(
    """               # #fff or #FBF01B
    \#                   # starts with #
    ([a-fA-F0-9]{3})     # excatly 3 hex digits
    $                    # and nothing more
    |                    # OR
    \#
    ([a-fA-F0-9]{6})     # exactly 6 hex digits
    $                    # and nothing more
    """, re.VERBOSE)


RGB_MATCH = re.compile(
    """                # rgb(0, 12, 123)
    ^[rR][gG][bB]      # starts with "rgb"
    \s*\(\s*           # spaces ( spaces
    \d{1,3}            # 1-3 digits
    \s*,\s*            # spaces comma spaces
    \d{1,3}            # 1-3 digits
    \s*,\s*            # spaces comma spaces
    \d{1,3}            # 1-3 digits
    \s*\)$             # spaces ) EOS
    """, re.VERBOSE)


RGBA_MATCH = re.compile(
    """                # rgba(0, 12, 123, 0.3)
    ^[rR][gG][bB][aA]  # starts with "rgb"
    \s*\(\s*           # spaces ( spaces
    \d{1,3}            # 1-3 digits
    \s*,\s*            # spaces comma spaces
    \d{1,3}            # 1-3 digits
    \s*,\s*            # spaces comma spaces
    \d{1,3}            # 1-3 digits
    \s*,\s*            # spaces comma spaces
    \d+                # first digit of opacity
    (\.\d+)?           # one or more digit after optional decimal place
    \s*\)$             # spaces ) EOS
    """, re.VERBOSE)


HSL_MATCH = re.compile(
    """                # hsl(120, 5%, 100%)
    ^[hH][sS][lL]      # starts with "hsl"
    \s*\(\s*           # spaces ( spaces
    \d{1,3}            # 1-3 digits for Hue
    \s*,\s*            # spaces comma spaces
    \d{1,3}\s*%        # 1-3 digits spaces "%" for Saturation
    \s*,\s*            # spaces comma spaces
    \d{1,3}\s*%        # 1-3 digits spaces "%" for Lightness
    \s*\)$             # spaces ) EOS
    """, re.VERBOSE)


HSLA_MATCH = re.compile(
    """                # hsla(120, 5%, 100%, 0.3)
    ^[hH][sS][lL][aA]  # starts with "hsla"
    \s*\(\s*           # spaces ( spaces
    \d{1,3}            # 1-3 digits for Hue
    \s*,\s*            # spaces comma spaces
    \d{1,3}\s*%        # 1-3 digits spaces "%" for Saturation
    \s*,\s*            # spaces comma spaces
    \d{1,3}\s*%        # 1-3 digits spaces "%" for Lightness
    \s*,\s*            # spaces comma spaces
    \d+                # first digits of opacity
    (\.\d+)?           # optional one or more decimal digits
    \s*\)$             # spaces ) EOS
    """, re.VERBOSE)


def is_colour(value):
    """Returns True if the value given is a valid CSS colour, i.e. matches one
    of the regular expressions in the module or is in the list of
    predetefined values by the browser.
    """
    global PREDEFINED, HEX_MATCH, RGB_MATCH, RGBA_MATCH, HSL_MATCH, HSLA_MATCH
    value = value.strip()

    # hex match
    if HEX_MATCH.match(value) or RGB_MATCH.match(value) or \
            RGBA_MATCH.match(value) or HSL_MATCH.match(value) or \
            HSLA_MATCH.match(value) or value in PREDEFINED:
        return True

    return False



# awl.tests.test_admintools.py
from django.test import TestCase
from django.contrib.admin.utils import label_for_field

from wrench.utils import parse_link

from awl.waelsteng import AdminToolsMixin
from awl.tests.models import Nested, Inner, Outer
from awl.tests.admin import InnerAdmin, OuterAdmin

# ============================================================================

class AdminToolsTest(TestCase, AdminToolsMixin):
    def test_admin_obj_mixin(self):
        # setup the admin site and id
        self.initiate()

        inner_admin = InnerAdmin(Inner, self.site)
        outer_admin = OuterAdmin(Outer, self.site)

        n1 = Nested.objects.create(name='n1')
        i1 = Inner.objects.create(name='i1', nested=n1)
        o1 = Outer.objects.create(name='o1', inner=i1)

        # check the basic __str__ named link from Inner to Nested
        html = self.field_value(inner_admin, i1, 'show_nested')
        url, text = parse_link(html)
        self.assertEqual('Nested(id=1 n1)', text)
        self.assertEqual('/admin/tests/nested/?id__exact=1', url)

        # check the template based name link of Inner from Outer
        html = self.field_value(outer_admin, o1, 'show_inner')
        url, text = parse_link(html)
        self.assertEqual('Inner.id=1', text)
        self.assertEqual('/admin/tests/inner/?id__exact=1', url)

        # check the double dereferenced Nested from Outer
        html = self.field_value(outer_admin, o1, 'show_nested')
        url, text = parse_link(html)
        self.assertEqual('Nested(id=1 n1)', text)
        self.assertEqual('/admin/tests/nested/?id__exact=1', url)

        # check the title got set correctly
        label = label_for_field('show_inner', o1, outer_admin)
        self.assertEqual(label, 'My Inner')

        # check that empty values work properly
        o2 = Outer.objects.create(name='o2')
        result = self.field_value(outer_admin, o2, 'show_nested')
        self.assertEqual('', result)

        i2 = Inner.objects.create(name='i2')
        o2.inner = i2
        o2.save()
        result = self.field_value(outer_admin, o2, 'show_nested')
        self.assertEqual('', result)
